/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.bizframe.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.tinygroup.bizframe.PermissionManager;
import org.tinygroup.bizframe.PermissionObject;
import org.tinygroup.bizframe.PermissionCheckStrategy;
import org.tinygroup.bizframe.PermissionObjectChecker;
import org.tinygroup.bizframe.PermissionSubject;
import org.tinygroup.bizframe.PermissionSubjectChecker;

//TODO，缓冲支持，权限主体、客体继承支持，需要在策略代码中加以实现
/**
 * 如果没有配置策略，那么就是允许即返回是否配置了允许，禁止即是否配置了禁止
 * 
 * @author luoguo
 * 
 */
@SuppressWarnings("rawtypes")
public class PermissionManagerImpl implements PermissionManager {

	private Map<String, PermissionSubject> permissionSubjectMap = new HashMap<String, PermissionSubject>();
	private Map<String, PermissionObject> permissionObjectMap = new HashMap<String, PermissionObject>();;
	private Map<String, HashSet<String>> allowPermissionMap = new HashMap<String, HashSet<String>>();;
	private Map<String, HashSet<String>> blockPermissionMap = new HashMap<String, HashSet<String>>();
	private PermissionCheckStrategy permissionStrategy = null;
	private boolean permissionSubjectInheritSupport;// 权限主体是否支持继承
	private boolean permissionObjectInheritSupport;// 权限客体是否支持继承
	private boolean cacheSupport;// 是否允许缓冲
	private List<PermissionSubjectChecker> permissionSubjectCheckerList;
	private List<PermissionObjectChecker> permissionObjectCheckerList;

	public boolean isBlock(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		if (permissionStrategy != null) {
			return permissionStrategy.isBlock(permissionSubject,
					permissionObject);
		} else {
			return isBlockDirectly(permissionSubject, permissionObject);
		}
	}

	public boolean isAllow(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		if (permissionStrategy != null) {
			return permissionStrategy.isAllow(permissionSubject,
					permissionObject);
		} else {
			return isAllowDirectly(permissionSubject, permissionObject);
		}
	}

	public boolean isBlock(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		if (permissionStrategy != null) {
			return permissionStrategy.isBlock(permissionSubjectType,
					permissionSubjectId, permissionObjectType,
					permissionObjectId);
		} else {
			return isBlockDirectly(permissionSubjectType, permissionSubjectId,
					permissionObjectType, permissionObjectId);
		}
	}

	public boolean isAllow(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		if (permissionStrategy != null) {
			return permissionStrategy.isAllow(permissionSubjectType,
					permissionSubjectId, permissionObjectType,
					permissionObjectId);
		} else {
			return isAllowDirectly(permissionSubjectType, permissionSubjectId,
					permissionObjectType, permissionObjectId);
		}
	}

	public void addPermissionSubject(PermissionSubject permissionSubject) {
		permissionSubjectMap.put(getPermissionSubjectKey(permissionSubject),
				permissionSubject);
	}

	private String getPermissionSubjectKey(PermissionSubject permissionSubject) {
		return permissionSubject.getType() + "-" + permissionSubject.getId();
	}

	public void addPermissionObject(PermissionObject permissionObject) {
		permissionObjectMap.put(getPermissionObjectKey(permissionObject),
				permissionObject);
	}

	private String getPermissionObjectKey(PermissionObject permissionObject) {
		return permissionObject.getType() + "-" + permissionObject.getId();
	}

	public void addAllowPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		addAllowPermission(permissionSubjectKey, permissionObjectKey);

	}

	public void addAllowPermission(
			List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList) {
		for (PermissionSubject permissionSubject : permissionSubjectList) {
			for (PermissionObject permissionObject : permissionObjectList) {
				addAllowPermission(permissionSubject, permissionObject);
			}
		}
	}

	public void removeAllowPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		removeAllowPermission(permissionSubjectKey, permissionObjectKey);
	}

	public void removeAllowPermission(
			List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList) {
		for (PermissionSubject permissionSubject : permissionSubjectList) {
			for (PermissionObject permissionObject : permissionObjectList) {
				removeAllowPermission(permissionSubject, permissionObject);
			}
		}
	}

	public void addAllowPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		addAllowPermission(permissionSubjectKey, permissionObjectKey);

	}

	public void removeAllowPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		removeAllowPermission(permissionSubjectKey, permissionObjectKey);

	}

	private void removeAllowPermission(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = allowPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList != null) {
			permissionObjectIdList.remove(permissionObjectKey);
		}
	}

	private void addAllowPermission(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = allowPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList == null) {
			permissionObjectIdList = new HashSet<String>();
			allowPermissionMap
					.put(permissionSubjectKey, permissionObjectIdList);
		}
		permissionObjectIdList.add(permissionObjectKey);
	}

	public void addBlockPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		addBlockPermission(permissionSubjectKey, permissionObjectKey);

	}

	public void addBlockPermission(
			List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList) {
		for (PermissionSubject permissionSubject : permissionSubjectList) {
			for (PermissionObject permissionObject : permissionObjectList) {
				addBlockPermission(permissionSubject, permissionObject);
			}
		}
	}

	public void removeBlockPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		removeBlockPermission(permissionSubjectKey, permissionObjectKey);
	}

	public void removeBlockPermission(
			List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList) {
		for (PermissionSubject permissionSubject : permissionSubjectList) {
			for (PermissionObject permissionObject : permissionObjectList) {
				removeBlockPermission(permissionSubject, permissionObject);
			}
		}
	}

	public void addBlockPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		addBlockPermission(permissionSubjectKey, permissionObjectKey);

	}

	public void removeBlockPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		removeBlockPermission(permissionSubjectKey, permissionObjectKey);

	}

	private void removeBlockPermission(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = blockPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList != null) {
			permissionObjectIdList.remove(permissionObjectKey);
		}
	}

	private void addBlockPermission(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = blockPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList == null) {
			permissionObjectIdList = new HashSet<String>();
			blockPermissionMap
					.put(permissionSubjectKey, permissionObjectIdList);
		}
		permissionObjectIdList.add(permissionObjectKey);
	}

	private String getKey(String type, String id) {
		return type + "-" + id;
	}

	public void setPermissionStrategy(PermissionCheckStrategy permissionStrategy) {
		permissionStrategy.setPermissionManager(this);
		this.permissionStrategy = permissionStrategy;
	}

	public boolean isPermissionObjectInheritSupport() {
		return permissionObjectInheritSupport;
	}

	public void setPermissionObjectInheritSupport(boolean support) {
		this.permissionObjectInheritSupport = support;

	}

	public boolean isPermissionSubjectInheritSupport() {
		return permissionSubjectInheritSupport;
	}

	public void setPermissionSubjectInheritSupport(boolean support) {
		this.permissionSubjectInheritSupport = support;
	}

	public boolean isCacheSupport() {
		return cacheSupport;
	}

	public void setCacheSupport(boolean support) {
		this.cacheSupport = support;
	}

	public boolean isBlockDirectly(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		return isBlockDirectly(permissionSubjectKey, permissionObjectKey);
	}

	private boolean isBlockDirectly(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = blockPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList == null) {
			return false;
		} else {
			return permissionObjectIdList.contains(permissionObjectKey);
		}
	}

	public boolean isAllowDirectly(PermissionSubject permissionSubject,
			PermissionObject permissionObject) {
		String permissionSubjectKey = getPermissionSubjectKey(permissionSubject);
		String permissionObjectKey = getPermissionObjectKey(permissionObject);
		return isAllowDirectly(permissionSubjectKey, permissionObjectKey);
	}

	private boolean isAllowDirectly(String permissionSubjectKey,
			String permissionObjectKey) {
		HashSet<String> permissionObjectIdList = allowPermissionMap
				.get(permissionSubjectKey);
		if (permissionObjectIdList == null) {
			return false;
		} else {
			return permissionObjectIdList.contains(permissionObjectKey);
		}
	}

	public boolean isBlockDirectly(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		return isBlockDirectly(permissionSubjectKey, permissionObjectKey);
	}

	public boolean isAllowDirectly(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId) {
		String permissionSubjectKey = getKey(permissionSubjectType,
				permissionSubjectId);
		String permissionObjectKey = getKey(permissionObjectType,
				permissionObjectId);
		return isAllowDirectly(permissionSubjectKey, permissionObjectKey);
	}

	public void setPermissionSubjectCheckerList(
			List<PermissionSubjectChecker> permissionSubjectCheckerList) {
		this.permissionSubjectCheckerList = permissionSubjectCheckerList;

	}

	public void setPermissionObjectCheckerList(
			List<PermissionObjectChecker> permissionObjectCheckerList) {
		this.permissionObjectCheckerList = permissionObjectCheckerList;

	}

	public boolean belongTo(PermissionSubject sourceObject,
			PermissionSubject destObject) {
		for (PermissionSubjectChecker checker : permissionSubjectCheckerList) {

			boolean sourceTypeMatch = checker.getPermissionSubjectType()
					.isInstance(sourceObject);
			boolean destTypeMatch = checker.getPermissionSubjectContainerType()
					.isInstance(destObject);
			if (sourceTypeMatch && destTypeMatch) {
				boolean result = checker.belongTo(sourceObject, destObject);
				if (result) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean contains(PermissionSubject sourceObject,
			PermissionSubject destObject) {
		for (PermissionSubjectChecker checker : permissionSubjectCheckerList) {

			boolean sourceTypeMatch = checker.getPermissionSubjectType()
					.isInstance(sourceObject);
			boolean destTypeMatch = checker.getPermissionSubjectContainerType()
					.isInstance(destObject);
			if (sourceTypeMatch && destTypeMatch) {
				boolean result = checker.contains(sourceObject, destObject);
				if (result) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean belongTo(PermissionObject sourceObject,
			PermissionObject destObject) {
		for (PermissionObjectChecker checker : permissionObjectCheckerList) {

			boolean sourceTypeMatch = checker.getPermissionObjectType()
					.isInstance(sourceObject);
			boolean destTypeMatch = checker.getPermissionObjectContainerType()
					.isInstance(destObject);
			if (sourceTypeMatch && destTypeMatch) {
				boolean result = checker.belongTo(sourceObject, destObject);
				if (result) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean contains(PermissionObject sourceObject,
			PermissionObject destObject) {
		for (PermissionObjectChecker checker : permissionObjectCheckerList) {

			boolean sourceTypeMatch = checker.getPermissionObjectType()
					.isInstance(sourceObject);
			boolean destTypeMatch = checker.getPermissionObjectContainerType()
					.isInstance(destObject);
			if (sourceTypeMatch && destTypeMatch) {
				boolean result = checker.contains(sourceObject, destObject);
				if (result) {
					return true;
				}
			}
		}
		return false;
	}

}
